package vsp;

import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimLink;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSecurityException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * Transaction generator
 * 
 * @author Jan Tichava <jtichava@students.zcu.cz>
 *
 */
public class Generator extends JSimProcess {
	
	/**
	 * Exponential distribution ratio
	 */
	private double lambda;
	/**
	 * Queue for server 1
	 */
	private QueueWithServer queue1;
	/**
	 * Queue for server 2
	 */
	private QueueWithServer queue2;
	
	/**
	 * Number of generated transactions
	 */
	private int generated;
	
	/**
	 * Creates new generator
	 * 
	 * @param name name of generator
	 * @param simulation simulation object
	 * @param lambda defined lambda
	 * @param queue1 queue for server 1
	 * @param queue2 queue for server 2
	 * @throws JSimSimulationAlreadyTerminatedException
	 * @throws JSimInvalidParametersException
	 * @throws JSimTooManyProcessesException
	 */
	public Generator(String name, JSimSimulation simulation, double lambda, QueueWithServer queue1, QueueWithServer queue2) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException {
		super(name, simulation);
		
		this.lambda = lambda;
		this.queue1 = queue1;
		this.queue2 = queue2;
		
		generated = 0;
	}
	
	/**
	 * In infinite loop:
	 * - generates new transaction
	 * - generates random number from <0; 1>
	 * - if number is smaller than queue 1 probability, than add transaction to queue 1, otherwise in queue 2
	 * - pause process in depend of lambda 
	 */
	protected void life() {
		try {
			
			while (true) {
				JSimLink link = new JSimLink(new Transaction(myParent.getCurrentTime()));
				
				if (JSimSystem.uniform(0.0, 1.0) < queue1.getProbability()) {
					queue1 = addTransaction(link, queue1);
				}
				else {
					queue2 = addTransaction(link, queue2);
				}

				generated++;
				
				hold(JSimSystem.negExp(lambda));
			}
			
		} catch (JSimSecurityException e) {
			e.printStackTrace();
		} catch (JSimInvalidParametersException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Add transaction to queue
	 * 
	 * @param link
	 * @param queue
	 * @return
	 * @throws JSimSecurityException
	 * @throws JSimInvalidParametersException
	 */
	private QueueWithServer addTransaction(JSimLink link, QueueWithServer queue) throws JSimSecurityException, JSimInvalidParametersException {
		
		link.into(queue);
		if (queue.getServer().isIdle()) {
			queue.getServer().activate(myParent.getCurrentTime());
		}
		
		return queue;
	}

	/**
	 * Get number of generated transaction
	 * 
	 * @return the generated
	 */
	public int getGenerated() {
		return generated;
	}
	
	public double getLq() {
		return myParent.getCurrentTime() / getGenerated();
	}
	
	
}
