package vsp;

import cz.zcu.fav.kiv.jsim.JSimHead;
import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimTooManyHeadsException;

/**
 * 
 * Queue with assigned server
 * 
 * @author Jan Tichava <jtichava@students.zcu.cz>
 *
 */
public class QueueWithServer extends JSimHead {

	/**
	 * Assigned server
	 */
	private JSimProcess server;
	/**
	 * Branch probability
	 */
	private double probability;
	
	/**
	 * 
	 * Creates new queue with defined server
	 * 
	 * @param name name of queue
	 * @param simulation simulation object
	 * @param probability probability of this branch
	 * @param server assigned server
	 * @throws JSimInvalidParametersException
	 * @throws JSimTooManyHeadsException
	 */
	public QueueWithServer(String name, JSimSimulation simulation, double probability, JSimProcess server) throws JSimInvalidParametersException, JSimTooManyHeadsException {
		super(name, simulation);
		this.server = server;
		this.probability = probability;
	}
	
	/**
	 * 
	 * Creates new queue without server definition
	 * 
	 * @param name name of queue
	 * @param simulation simulation object
	 * @param probability probability of this branch
	 * @throws JSimInvalidParametersException
	 * @throws JSimTooManyHeadsException
	 */
	public QueueWithServer(String name, JSimSimulation simulation, double probability) throws JSimInvalidParametersException, JSimTooManyHeadsException {
		this(name, simulation, probability, null);
	}

	/**
	 * Get server
	 * 
	 * @return the server
	 */
	public JSimProcess getServer() {
		return server;
	}

	/**
	 * Set server
	 * 
	 * @param server the server to set
	 */
	public void setServer(JSimProcess server) {
		this.server = server;
	}

	/**
	 * Get probability of this branch
	 * 
	 * @return the probability
	 */
	public double getProbability() {
		return probability;
	}
	
}
