package vsp;

import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimLink;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSecurityException;
import cz.zcu.fav.kiv.jsim.JSimSimulation;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimSystem;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * 
 * Server class
 * 
 * @author Jan Tichava <jtichava@students.zcu.cz>
 *
 */
public class Server extends JSimProcess {

	/**
	 * Mean value
	 */
	private double ts;
	/**
	 * Variation of the mean value
	 */
	private double cs;
	/**
	 * Assigned queue
	 */
	private QueueWithServer queue;
	
	/**
	 * Transactions counter
	 */
	private int counter;
	/**
	 * Real Tq
	 */
	private double transTq;
	

	/**
	 * 
	 * Creates new server
	 * 
	 * @param name server name
	 * @param simulation simulation object
	 * @param ts defined Ts
	 * @param cs defined Cs
	 * @param queue assigned input queue
	 * @throws JSimSimulationAlreadyTerminatedException
	 * @throws JSimInvalidParametersException
	 * @throws JSimTooManyProcessesException
	 */
	public Server(String name, JSimSimulation simulation, double ts, double cs, QueueWithServer queue) throws JSimSimulationAlreadyTerminatedException, JSimInvalidParametersException, JSimTooManyProcessesException {
		super(name, simulation);
		
		this.ts = ts;
		this.cs = cs;
		this.queue = queue;
		
		counter = 0;
		transTq = 0.0;
	}
	
	/**
	 * In infinite loop:
	 * - checks if queue is not empty
	 * - waits time Ts with some Cs and then removes transaction from queue
	 * 
	 */
	protected void life() {
		
		try {
			while(true) {
				if (queue.empty()) {
					passivate();
				}
				else {
					double mu = 1 / (JSimSystem.gauss(1/ts, cs/ts));
					hold(JSimSystem.negExp(mu));

					JSimLink link = queue.first();
					
					Transaction transaction = (Transaction) link.getData();
					counter++;
					transTq += myParent.getCurrentTime() - transaction.getCreationTime();
					link.out();
				}
			}
		} catch (JSimSecurityException e) {
			e.printStackTrace();
		} catch (JSimInvalidParametersException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Get count of managed transactions
	 * 
	 * @return the counter
	 */
	public int getCounter() {
		return counter;
	}

	/**
	 * Get global Tq
	 * 
	 * @return the transTq
	 */
	public double getTransTq() {
		return transTq;
	}

	/**
	 * Get average Tq for one request
	 * 
	 * @return Tq
	 */
	public double getTq() {
		return transTq/counter;
	}
}
